var LoginController = function(ScreenService, templateManager, LoginService, LoginUtilities, localStorageEngine, eventAggregator, requestDataSourceEngine, restEngine, MessageService, ZHPromise, ZhuAclController, ZhLogger, metaDataFactory, ZhKeyboardService) {
    
    /* global angular */
    /* global Configuration */
    /* global localStorage */
    /* global setLocalVar*/
    /* global Util */
    
    const self = this;
    let loginUserId, loginUserName;
    
    this.authenticationContainer = 'product#Login';
    this.changePasswordContainer = "product#AlterarSenha"
    
    /*****************************/
    /***** Starting Function *****/
    /*****************************/
    
    this.authenticate = function(forceAuthenticate) {
        check = self.checkChangePasswordContainer();
        
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        self.updateLanguage();

        const defer = ZHPromise.defer();
        
        if (!check) {
            forceAuthenticate = forceAuthenticate === true;
            if (window.location.hash.replace('#/', '') != self.authenticationContainer || forceAuthenticate) {
                const redirectUrl = localStorage.getItem('LOGIN_REDIRECT_URL');
                const user = localStorage.getItem('LOGIN_USER');
                const authToken = self.updateToken();
    
                if (redirectUrl && !redirectUrl.includes(window.location.hostname)) {
                    self.redirectToRedirectionProduct(redirectUrl, defer);
                } else if (!user && !authToken) {
                    self.redirectToLogin();
                    setLocalVar('INICIA_DASHBOARD', false);
                    defer.resolve({});
                } else {
                    let setAclEventId;
                    const setAclFunction = function () {
                        self.setAcl(defer.promise, setAclEventId);
                    };
                    
                    setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);
    
                    self.startSession(user).then(function (response) {
                        if (response.ERROR == true) {
                            MessageService.showMessageByCode("product#" + response.MESSAGE).then(function() {
                                self.logout();
                            });
                        } else {
                            self.startSessionResponse(response, defer);
                        }                     
                    });
                }
            }
        } else {
            defer.resolve({});
        }
        
        return defer.promise;
    };
    
    this.startLoginProcess = function (widgetName) {
        const widget = templateManager.container.getWidget(widgetName);

        const data = self.getDataFromWidget(widget);
        const user = data.USER;
        const password = data.PASSWORD;

        if (!user) {
            MessageService.showMessageByCode('product#ALE_NOT_EMAIL', null, { type: 'ALERT' });
        } else if (!password) {
            MessageService.showMessageByCode('product#ALE_NOT_PASSWORD', null, { type: 'ALERT' });
        } else {
            if (!widget.attempts.hasOwnProperty(user)) {
                widget.attempts[user] = 1;
            }
            
            const row = self.createLoginRowObject(user, password, widget.attempts[user]);
            
            LoginService.requestRow("product#/loginSaaS", row).then(function(response) {
                const callback = response.dataset.request;
                
                if (callback.ERROR == true) {
                    widget.attempts[user]++;
                    MessageService.showMessageByCode("product#" + callback.MESSAGE);
                } else {
                    widget.currentRow = {};
                    self.redirectAfterLogin(callback);
                }
            });
        }
    };
    
    this.getDataFromWidget = function(widget) {
        const data = {};
        
        switch (widget.name) {
            case "LOGIN":
                data.USER = widget.currentRow.USER;
                data.PASSWORD = widget.currentRow.PASSWORD;
            break;
            
            case "CHANGE_PASSWORD":
                data.USER = self.getParameterByName("email");
                data.PASSWORD = widget.currentRow.NEW_PASSWORD;
            break;
        }
        
        return data;
    };
    
    this.createLoginRowObject = function(user, password, attempts) {
        return {
            "USER_EMAIL": user, 
            "PASSWORD": password, 
            "PRODUCT_ID": window.projectId, 
            "REQUESTER_URL": document.location.href, 
            "ATTEMPTS": attempts, 
            "SHOW_FULL_OPERATOR": window.showFullOperatorWindow || false
        };
    };
    
    /******************/
    /***** LOGOUT *****/
    /******************/
    
    this.logout = function() {
        LoginService.requestRow("product#/logoutSaaS", {}).then(function () {
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            self.redirectToLogin();
        });
    };
    
    /*********************/
    /***** REDIRECTS *****/
    /*********************/
    
    this.redirectToRedirectionProduct = function(redirectUrl, defer) {
        if (window.ZhNativeInterface) {
            window.ZhNativeInterface.redirect(redirectUrl);
        } else {
            window.location = redirectUrl;
        }
        
        defer.resolve({});
    };
    
    this.redirectToProduct = function() {
        const containerName = window.defaultContainer || templateManager.project.mainWindow;
        
        if (window.openLoginContainerWithoutRedirect) {
            eventAggregator.publish('loginSuccessWithoutRedirect');
            ScreenService.openWindow(containerName);
        } else {
            self.openWindow(containerName);
        }
    };
    
    this.redirectToLogin = function() {
        if (window.openLoginContainerWithoutRedirect) {
            ScreenService.openWindow(self.authenticationContainer);
        } else {
            self.openWindow(self.authenticationContainer);
        }
    };
    
    this.redirectAfterLogin = function(loginData) {
        localStorage.setItem('LOGIN_USER', loginData.USER);
        localStorage.setItem('LOGIN_TOKEN', loginData.TOKEN);

        if (loginData.REDIRECT_URL) {
            loginData.REDIRECT_URL = self.validateRcUrl(loginData.REDIRECT_URL);
            localStorage.setItem('LOGIN_REDIRECT_URL', loginData.REDIRECT_URL);
            loginData.REDIRECT_URL += '?authToken=' + loginData.REDIRECTION_TOKEN;
            
            if (window.ZhNativeInterface) {
                window.ZhNativeInterface.redirect(loginData.REDIRECT_URL);
            } else {
                window.location = loginData.REDIRECT_URL;
            }
        } else {
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            self.redirectToProduct();
        }
    };
    
    /********************************/
    /***** FUNCTIONS CONTROLLER *****/
    /********************************/
    
    this.validateRcUrl = function(redirectUrl) {
        const url = window.location.href;
        
        if (url.includes('rc-')) {
            let redirectHost = angular.copy(redirectUrl);
            
            if (redirectHost.includes('://')) {
                redirectHost = redirectHost.split('://')[1];
            }
            
            let protocol = '';
            
            if (redirectUrl.includes('https')) {
                protocol = 'https://';
            } else if (redirectUrl.includes('http')) {
                protocol = 'http://';
            }
            
            redirectUrl = protocol + 'rc-' + redirectHost;
        }
        
        return redirectUrl;
    };
    
    this.updateToken = function () {
        const authToken = Util.getUrlParameter('authToken');
        
        if (authToken) {
            localStorage.setItem('LOGIN_TOKEN', authToken);
        }
        
        return authToken;
    };
    
    this.setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if (response.ERROR == true) {
                MessageService.showMessageByCode("product#" + response.MESSAGE).then(function() {
                    self.logout();
                });
            } else {
                if (!window.productApplyAcl) {
                    ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                        {
                            canConfigureAcl: response.ACL_ADMINISTRATOR,
                            rules          : response.ACL_ADMINISTRATOR ? [] : false,
                            defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                        }
                    );
                    
                    templateManager.project.aclDefaultRule = 'A';
                }
                
                eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
            }
        });
    };
    
    this.startSessionResponse = function(response, defer) {
        if (response.ERROR) {
            defer.reject(response.MESSAGE);
        } else {
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                self.logoutExpiredSession(true);
            } else {
                localStorage.setItem('IS_LOGGED', true);
                
                if(response.USER && response.TOKEN) {
                    localStorage.setItem('LOGIN_USER', response.USER);
                    localStorage.setItem('LOGIN_TOKEN', response.TOKEN);
                }

                loginUserId = response.USER_ID;
                loginUserName = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    MessageService.showMessageByCode('product#ALE_MIGRATION_IS_PENDENT', null, { type: 'INFORMATION' });
                }

                if (response.MODULES_VERSION_ERROR) {
                    if(response.MODULES_VERSION_ERROR.MESSAGE_CODE) {
                        const modulesMessageCode = response.MODULES_VERSION_ERROR.MESSAGE_CODE;
                        const modulesMessageParams = response.MODULES_VERSION_ERROR.MESSAGE_PARAMS;
                        MessageService.getMessageByCode('lib#' + modulesMessageCode).then(function (message) {
                            ZhLogger.warning(ScreenService.i18n(message, modulesMessageParams));
                        });
                    } else {
                        ZhLogger.warning(response.MODULES_VERSION_ERROR);
                    }
                }

                defer.resolve(response);
            }
        }
    };
    
    this.logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        const message = afterLogin ? 'ERR_EXPIRED_SESSION_LOGIN' : 'ERR_ATTEMPT_END';
        
        MessageService.getMessageByCode('product#' + message).then(function(loadingMessage) {
            ScreenService.changeLoadingMessage(ScreenService.i18n(loadingMessage));
            ScreenService.showLoader();
            Util.doAsync(self.logout, 5000);
        });
    };
    
    this.openWindow = function(windowName, newWindow) {
        return self.requestModulesMetadata().then(function (modules) {
            const subModule = windowName.split('#')[0];
            const module = subModule.split('_')[0];
            const url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
            
            eventAggregator.publish("onFirstLogin");
        });
    };
    
    this.requestModulesMetadata = function () {
        const modulesPackage = window.modulesPackage || "modules";
        
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };
    
    this.startSession = function(user) {
        const row = {
            'USER': user,
            'IS_LOGGED': localStorage.getItem('IS_LOGGED') || false,
            'LOGIN_TOKEN': localStorage.getItem('LOGIN_TOKEN')
        }; 

        return LoginService.requestRow('product#/checkActiveSession', row, 1, undefined, undefined, undefined, 120000).then(function(response) {
            const callback = response.dataset.request;
            
            if (callback.ERROR == true) {
                MessageService.showMessageByCode('product#' + callback.MESSAGE);    
            } else {
                return callback;
            }
        });
    };
    
	this.changeLoginResponseToOldParams = function(loginResponse) {
        if(loginResponse) {
            return {
                cdoperador:          loginResponse.USER_ID,
                nmoperador:          loginResponse.USER_NAME,
                dsemailoper:         loginResponse.USER_EMAIL,
                nrorg:               loginResponse.ORGANIZATION_ID,
                nmorg:               loginResponse.TRADING_NAME,
                nmrazsocialorg:      loginResponse.COMPANY_NAME,
                perfilDeAcesso:      loginResponse.ACCESS_PROFILE,

                cdempresa:           loginResponse.CDEMPRESA     || null,
                cdfilialselecionada: loginResponse.CDFILIAL      || null,
                cdpictprat:          loginResponse.CDPICTPRAT    || null,
                cdpictprod:          loginResponse.CDPICTPROD    || null,
                ididioma:            loginResponse.IDIDIOMA      || null,
                nmacesso:            loginResponse.NMACESSO      || null,
                nmcomploper:         loginResponse.NMCOMPLOPER   || null,
                nmfilialselecionada: loginResponse.NMFILIAL      || null,
                nrestruttrab:        loginResponse.NRESTRUTTRAB  || null,
                nrorgtrab:           loginResponse.NRORGTRAB     || null,
                nrparcnegocio:       loginResponse.NRPARCNEGOCIO || null,

                logged:              loginResponse.LOGGED,
                senhaExpirada:       loginResponse.EXPIRED_PASSWORD,
                version:             loginResponse.VERSION
            };
        }
    };
    
    /**************************/
    /**** LANGUAGE EVENTS *****/
    /**************************/
    
    this.updateLanguage = function () {
        const language = localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);
    };
    
    this.changeSystemLanguage = function (widgetName) {
        const widget = templateManager.container.getWidget(widgetName);
        const actualLanguage = localStorage.getItem('LOGIN_LANGUAGE');
        const language = widget.currentRow.LANGUAGE;
        
        if (actualLanguage != language) {
            localStorage.setItem('LOGIN_LANGUAGE', language);
            
            ScreenService.setLanguage(language).then(function() {
                let updatedTitle = ScreenService.i18n(templateManager.container.label);
                
                if (document.title.includes(' - ')) {
                    const productSufixTitle = document.title.split(' - ').pop();
                    updatedTitle = updatedTitle + ' - ' + productSufixTitle;
                }
                
                templateManager.updateTitle(updatedTitle);
            });
        }
    };
    
    this.updateLanguageField = function(widget) {
        const languageField = widget.getField("LANGUAGE");
        let language = window.lang;
        ScreenService.showLoader();
        
        const filter = [ 
            {
                name: 'PRODUCT_ID',
                operator: '=',
                value: window.projectId
            } 
        ];
        
        LoginService.requestFilter("product#/productLanguages", filter).then(function(response) {
            ScreenService.hideLoader();
            
            const productLanguages = response.dataset.languages.LANGUAGES;
            const defaultLanguage  = response.dataset.languages.DEFAULT_LANGUAGE;
            
            if (defaultLanguage && defaultLanguage.SHORT_NAME) {
                const defaultLangValidation = productLanguages.filter(function(lang) {
                    return lang.SHORT_NAME == defaultLanguage.SHORT_NAME;
                });
                
                if (Util.isEmptyOrBlank(defaultLangValidation)) {
                    productLanguages.push(defaultLanguage);
                }
                
                language = localStorage.getItem('LOGIN_LANGUAGE') || defaultLanguage.SHORT_NAME;
            } else {
                language = localStorage.getItem('LOGIN_LANGUAGE') || language;
            }
            
            if (!Util.isEmptyOrBlank(productLanguages)) {
                languageField.dataSource.data = productLanguages;
                languageField.readOnly = productLanguages.length == 1;
            } else {
                languageField.readOnly = true;
            }
            
            self.setCurrentLanguage(language, widget, languageField);
        }).catch(function() {
            ScreenService.hideLoader();
            languageField.readOnly = true;
            self.setCurrentLanguage(language, widget, languageField);
        });
    };
    
    this.setCurrentLanguage = function(language, widget, languageField) {
        localStorage.setItem('LOGIN_LANGUAGE', language);
        
        ScreenService.setLanguage(language).then(function() {
            languageField.value(language);
            widget.redraw();
        });
    };
    
    /****************************/
    /**** LOGIN JSON EVENTS *****/
    /****************************/
    
    this.removeAdditionalContainers = function() {
        const containers = templateManager.containers;
        
        templateManager.project.aclDefaultRule = 'A';
        templateManager.aclRules = {};
        
        for (let key in containers) {
            if (key.includes('#') && !key.includes('product#Login') && !key.includes('product#AlterarSenha')) {
                delete containers[key];
            }
        }
    };
    
    this.setDefaults = function () {
        if (localStorage.getItem('LOGIN_USER')) {
            self.redirectToProduct();
        } else {
            if (ScreenService.isSideMenuOpen()) {
                templateManager.container.showMenu = true;
                ScreenService.toggleSideMenu();
                templateManager.container.showMenu = false;
            }
            
            const widget = templateManager.container.getWidget("LOGIN");
            
            widget.attempts = {};
            widget.getField('KEEP_CONNECTED').value(localStorage.getItem('LOGIN_KEEP_CONNECTED'));
            self.updateLanguageField(widget);
            ZhKeyboardService.bind('ctrl+shift+alt+,', self.testLicenseConnection);
        }
    };
    
    this.openPopUpForgotPassword = function () {
        const widget = templateManager.container.getWidget("FORGOT_PASSWORD");
        
        ScreenService.openPopup(widget);
    };
    
    this.setKeepConnected = function () {
        const widget = templateManager.container.getWidget("LOGIN");
        
        localStorage.setItem('LOGIN_KEEP_CONNECTED', widget.currentRow.KEEP_CONNECTED);
    };
    
    this.testLicenseConnection = function() {
        LoginService.requestFilter("product#/testLicenseConnection", []).then(function (response) {
            const result = response.dataset.testLicenseConnection;
            const params = { URL: result.URL };
            
            if (result.ERROR) {
                params.ERROR = result.ERROR;
                MessageService.showMessageByCode('product#ERR_LICENSE_CONNECTION', params, {type: 'ERROR'});
            } else {
                MessageService.showMessageByCode('product#SUC_LICENSE_CONNECTION', params, {type: 'SUCCESS'});
            }
        }).catch(LoginUtilities.defaultError);
    };
    
    /*********************************/
    /***** URL DATA MANIPULATION *****/
    /*********************************/
    
    this.checkChangePasswordContainer = function() {
        const url = decodeURIComponent(window.location.hash.replace('#/', ''));
        const regex = new RegExp('[a-zA-Z]+#[a-zA-Z]+');
        const result = regex.exec(url);
        
        if (!result || result[0] != self.changePasswordContainer) {
            return false;
        } else {
            return true;
        }
    };
    
    this.getParameterByName = function(name, url) {
        if (!url) {
            url = decodeURIComponent(window.location.href);
        }
        
        name = name.replace(/[\[\]]/g, '\\$&');
        
        const regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);
            
        if (!results) { 
            return null;
        }
        
        if (!results[2]) {
            return '';
        }
        
        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    };
    
    /**********************************/
    /***** FORGOT PASSWORD WIDGET *****/
    /**********************************/
    
    this.setDefaultEmailForgotPassword = function () {
        const widgetLogin = templateManager.container.getWidget("LOGIN");
        const widgetForgotPassword = templateManager.container.getWidget("FORGOT_PASSWORD");
        
        widgetForgotPassword.newRow();
        widgetForgotPassword.currentRow.EMAIL = widgetLogin.currentRow.USER;
    };
    
    this.confirmForgotPassword = function () {
        const widget = templateManager.container.getWidget("FORGOT_PASSWORD");
        
        if (widget.isValid()) {
            const email = widget.currentRow.EMAIL;
            
            const row = {
                'EMAIL': email
            };
            
            LoginService.requestRow("product#/requestNewPassword", row).then(function (response) {
                const callback = response.dataset.request;
                
                if (callback.ERROR == true) {
                    MessageService.showMessageByCode('product#' + callback.MESSAGE);    
                } else {
                    MessageService.showMessageByCode('product#' + callback.MESSAGE, { "PARAM": email });
                    ScreenService.closePopup();
                }
            });
        }
    };
    
    /***************************************/
    /***** REDEFINE PASSWORD CONTAINER *****/
    /***************************************/
    
    this.validateLinkStillActive = function(widget) {
        const row = {
            "TOKEN": self.getParameterByName("token"),
            "EMAIL": self.getParameterByName("email")
        };
        
        LoginService.requestRow("product#/validateLinkStillActive", row).then(function (response) {
            const callback = response.dataset.request;
            
            if (callback.ERROR == true) {
                MessageService.showMessageByCode("product#" + callback.MESSAGE).then(function() {
                    self.redirectToLogin();
                });
            } else {
                self.updateLanguageField(widget);
            }
        });
    };
    
    this.changePassword = function() {
        const widget = templateManager.container.getWidget("CHANGE_PASSWORD");
        
        if (widget.isValid()) {
            const password = widget.currentRow.NEW_PASSWORD;
            const passwordConfirmation = widget.currentRow.NEW_PASSWORD_CONFIRMATION;
            
            const validation = self.checkPassword(password, passwordConfirmation);
            
            if (validation.ERROR == true) {
                MessageService.showMessageByCode('product#' + validation.MESSAGE, null, { type: 'ALERT' });
            } else {
                const row = {
                    "TOKEN": self.getParameterByName("token"),
                    "EMAIL": self.getParameterByName("email"),
                    "PASSWORD": widget.currentRow.NEW_PASSWORD,
                    "PASSWORD_CONFIRMATION": widget.currentRow.NEW_PASSWORD_CONFIRMATION
                };
                
                LoginService.requestRow("product#/redefinePassword", row).then(function (response) {
                    var callback = response.dataset.request;
                    
                    if (callback.ERROR == true) {
                        MessageService.showMessageByCode("product#" + callback.MESSAGE);
                    } else {
                        MessageService.showMessageByCode("product#" + callback.MESSAGE).then(function() {
                            self.redirectToLogin();
                        });
                    }
                });
            }
        }
    };
    
    this.checkPassword = function(password, passwordConfirmation) {
        if (password === passwordConfirmation) {
            const lowercase = self.getMatch(password, /[a-z]/g);
            const uppercase = self.getMatch(password, /[A-Z]/g);
            const number = self.getMatch(password, /[0-9]/g);
            const specialChars = self.getMatch(password, /[^\w]/g);
            
            if (lowercase >= 3 && uppercase >= 2 && number >= 2 && specialChars >= 1 && (password.length >= 8 && password.length <= 12)) {
                return {"ERROR": false};
            } else {
                return {"ERROR": true, "MESSAGE": "ERR_PASSWORD_REQ_MISS"};
            }
        } else {
            return {"ERROR": true, "MESSAGE": "ERR_PASSWORD_NOT_MATCH"};
        }
    };
    
    this.getMatch = function(text, regex) {
        const match = text.match(regex);
        
        return Array.isArray(match) ? match.length : 0;
    };
};

Configuration(function(ContextRegister) {
	ContextRegister.register('LoginController', LoginController);
});